#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include <nlist.h>
#include <unistd.h>
#include <signal.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <net/if.h>

#include <X11/Intrinsic.h>
#include <X11/Xatom.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

#include <X11/Xaw/Cardinals.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Paned.h>
#include "MemStripChart.h"
#include <X11/Xaw/StripChart.h>
#include <X11/Xmu/SysUtil.h>

#include "xnet.bit"

#define STRLENGTH 10

typedef struct _XResources {
  Boolean show_label;
  char *interface;
  int scale;
} XResources;

static XrmOptionDescRec options[] = {
 {"-scale",		"*xnet.scale",		XrmoptionSepArg,	"10"},
 {"-minscale",		"*xnet.minScale",	XrmoptionSepArg,	NULL},
 {"-update",		"*xnet.update",		XrmoptionSepArg,	NULL},

/*
 * For some reason, the -hl and -highlight command line options do not work 
 */
 {"-hl",		"*xnet*highlight",	XrmoptionSepArg,	NULL},
 {"-highlight",		"*xnet*highlight",	XrmoptionSepArg,	NULL},

 {"-jumpscroll",	"*xnet.jumpScroll",	XrmoptionSepArg,	NULL},
 {"-label",		"*label.label",		XrmoptionSepArg,	NULL},
 {"-interface",		"*interface",		XrmoptionSepArg,	"lo"},
 {"-nolabel",		"*showLabel",	        XrmoptionNoArg,       "False"},
};

static XtResource my_resources[] = {
  {"showLabel", XtCBoolean, XtRBoolean, sizeof(Boolean),
   XtOffsetOf(XResources, show_label), XtRImmediate, (XtPointer) TRUE},
  {"interface", XtCString, XtRString, sizeof(char *),
   XtOffsetOf(XResources, interface), XtRImmediate, (String)"eth0"},
  {"scale", XtNx, XtRInt, sizeof(int),
   XtOffsetOf(XResources, scale), XtRImmediate, (XtPointer)10},
};

static XResources resources;

void quit(Widget w, XEvent *event, String *params, Cardinal *num_params);

static XtActionsRec x_actions[] = {
	{ "quit", quit },
};

static Atom wm_delete_window;

void usage(char *progname) {
	fprintf(stderr, "usage: %s [-options ...]\n\n", progname);
	fprintf(stderr, "where options include:\n");
	fprintf(stderr, " -display dpy        X server on which to display\n");
	fprintf(stderr, " -geometry geom      size and location of window\n");
	fprintf(stderr, " -fn font            font to use in label\n");
	fprintf(stderr, " -scale number       scaling factor (> 0)\n");
	fprintf(stderr, " -minscale number    minimum scale lines to display\n");
	fprintf(stderr, " -update seconds     interval between updates\n");
	fprintf(stderr, " -label string       annotation text\n");
	fprintf(stderr, " -bg color           background color\n");
	fprintf(stderr, " -fg color           graph and text color\n");
	fprintf(stderr, " -hl color           scale color\n");
	fprintf(stderr, " -nolabel            remove the label from the chart\n");
	fprintf(stderr, " -jumpscroll value   number of pixels to scroll on overflow\n");
	fprintf(stderr, " -interface name     name of the interface to monitor\n");
	fprintf(stderr, "\n");
	exit(1);
}

void quit(Widget w, XEvent *event, String *params, Cardinal *num_params) {
	if ((event->type == ClientMessage) &&
		(event->xclient.data.l[0] != wm_delete_window)) {
		XBell(XtDisplay(w), 0);
		return;
		}

	exit(0);
}

void GetPoint(Widget w, caddr_t closure, caddr_t call_data) {
	double value = 0;
	FILE *F1 = NULL;
	char face[10] = { '\0' };
	int packets[2] = { 0, 0 };
	static int last_packets[2] = { 0, 0 };
	static char flag = 1;
	int skip = 0;
	char line[256] = { '\0' };
	char *word = NULL;

	F1 = fopen("/proc/net/dev", "r");
	if (F1 == NULL) {
		perror("/proc/net/dev\n");
		value = 0;
		}
	else {
		/*
		fgets(line, 256, F1);
		fgets(line, 256, F1);
		*/
		strcpy(face, "");
		while (!feof(F1)) {
			fgets(line, 256, F1);

			if (feof(F1))
				break;

			word = strtok(line, ": ");
			if (word != NULL) {
				strcpy(face, word);
				}

			word = strtok(NULL, " ");
			if (word != NULL) {
				sscanf(word, "%d", &packets[0]);
				}

			for (skip=0; skip < 4; skip++) {
				word = strtok(NULL, " ");
				}

			word = strtok(NULL, " ");
			if (word != NULL) {
				sscanf(word, "%d", &packets[1]);
				}

			if (!strcmp(face, resources.interface)) {
				value = ((double)packets[0] + 
					(double)packets[1] + 1.0);
				value -= ((double)last_packets[0] + 
					(double)last_packets[1] + 1.0);
				value /= resources.scale;

				last_packets[0] = packets[0];
				last_packets[1] = packets[1];
				}
			}
		}
	fclose(F1);

	*(double *)call_data = value;

	if (flag) {
		*(double *)call_data = 0;
		flag = 0;
		}
}

void main(int argc, char *argv[]) {
	XtAppContext app_con;
	Widget toplevel, net, pane, label_wid, net_parent;
	Arg args[1];
	Pixmap icon_pixmap = None;
	char *label, host[256];
	
	/* For security reasons (?) */
	setgid(getgid());
	setuid(getuid());

	toplevel = XtAppInitialize(&app_con, "Xnet", options,
		XtNumber(options), &argc, argv, NULL, NULL, (Cardinal)0);

	if (argc != 1)
		usage(argv[0]);

	XtGetApplicationResources( toplevel, (XtPointer)&resources,
		my_resources, XtNumber(my_resources), NULL, (Cardinal)0);

	XtAppAddActions(app_con, x_actions, XtNumber(x_actions));
	XtOverrideTranslations(toplevel,
		XtParseTranslationTable("<Message>WM_PROTOCOLS: quit()"));

	XtSetArg(args[0], XtNiconPixmap, &icon_pixmap);
	XtGetValues(toplevel, args, ONE);
	if (icon_pixmap == None) {
		XtSetArg(args[0], XtNiconPixmap,
			XCreateBitmapFromData(XtDisplay(toplevel),
				XtScreen(toplevel)->root,
				(char *)xnet_bits,
				xnet_width, xnet_height));
		XtSetValues(toplevel, args, ONE);
		}

	if (resources.show_label) {
		pane = XtCreateManagedWidget("paned", panedWidgetClass,
			toplevel, NULL, ZERO);

		label_wid = XtCreateManagedWidget("label", labelWidgetClass,
			pane, NULL, ZERO);

		XtSetArg(args[0], XtNlabel, &label);
		XtGetValues(label_wid, args, ONE);

		if (strcmp("label", label) == 0) {
			XmuGetHostname(host, 255);
			XtSetArg(args[0], XtNlabel, host);
			XtSetValues(label_wid, args, ONE);
			}

		net_parent = pane;
		}
	else
		net_parent = toplevel;

	net = XtCreateManagedWidget("net", stripChartWidgetClass,
		net_parent, NULL, ZERO);
	XtAddCallback(net, XtNgetValue, (void *)GetPoint, NULL);

	XtRealizeWidget(toplevel);
	wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW",
		False);

	XSetWMProtocols(XtDisplay(toplevel), XtWindow(toplevel),
		&wm_delete_window, 1);

	XtAppMainLoop(app_con);
}
